/*
    scconfig - detection of standard library features: time/date/sleep related calls
    Copyright (C) 2011..2012  Tibor Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

		Project page: http://repo.hu/projects/scconfig
		Contact via email: scconfig [at] igor2.repo.hu
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "libs.h"
#include "log.h"
#include "db.h"
#include "dep.h"

static int try(int logdepth, const char *test_c, char *cflags, char *ldflags)
{
	char *out = NULL;

	logprintf(logdepth, "trying lib with ldflags '%s'\n", ldflags == NULL ? get("cc/ldflags") : ldflags);
	if (compile_run(logdepth+1, test_c, NULL, cflags, ldflags, &out) == 0) {
		if (target_emu_fail(out) || (strncmp(out, "OK", 2) == 0)) {
			free(out);
			return 1;
		}
		free(out);
	}
	return 0;
}


int find_time_usleep(int logdepth, int fatal)
{
	char *test_c =
		NL "#include <unistd.h>"
		NL "int main() {"
		NL "	if (usleep(1) == 0)"
		NL "		puts(\"OK\");"
		NL "	return 0;"
		NL "}"
		NL;

	require("cc/cc", logdepth, fatal);

	report("Checking for usleep()... ");
	logprintf(logdepth, "find_time_usleep: trying to find usleep...\n");
	logdepth++;

	if (try(logdepth, test_c, NULL, NULL)) {
		put("libs/time/usleep/presents", strue);
		report("OK\n");
		return 0;
	}
	put("libs/time/usleep/presents", sfalse);
	report("not found.\n");

	return 1;
}

int find_time_Sleep(int logdepth, int fatal)
{
	char *test_c =
		NL "#include <windows.h>"
		NL "int main() {"
		NL "	Sleep(1);"
		NL "	puts(\"OK\");"
		NL "	return 0;"
		NL "}"
		NL;

	require("cc/cc", logdepth, fatal);

	report("Checking for Sleep()... ");
	logprintf(logdepth, "find_time_Sleep: trying to find Sleep...\n");
	logdepth++;

	if (try(logdepth, test_c, NULL, NULL)) {
		put("libs/time/Sleep/presents", strue);
		put("libs/time/Sleep/includes", "#include \"windows.h\"");
		report("OK\n");
		return 0;
	}
	put("libs/time/Sleep/presents", sfalse);
	report("not found.\n");

	return 1;
}

int find_time_gettimeofday(int logdepth, int fatal)
{
	char *test_c =
		NL "#include <stdlib.h>"
		NL "#include <sys/time.h>"
		NL "int main() {"
		NL "	struct timeval tv;"
		NL "	if (gettimeofday(&tv, NULL) == 0)"
		NL "		puts(\"OK\");"
		NL "	return 0;"
		NL "}"
		NL;

	require("cc/cc", logdepth, fatal);

	report("Checking for gettimeofday()... ");
	logprintf(logdepth, "find_time_gettimeofday: trying to find gettimeofday...\n");
	logdepth++;

	if (try(logdepth, test_c, NULL, NULL)) {
		put("libs/time/gettimeofday/presents", strue);
		report("OK\n");
		return 0;
	}
	put("libs/time/gettimeofday/presents", sfalse);
	report("not found.\n");

	return 1;
}


int find_time_ftime(int logdepth, int fatal)
{
	char *test_c =
		NL "#include <sys/timeb.h>"
		NL "int main() {"
		NL "	struct timeb tb;"
		NL "	if (ftime(&tb) == 0)"
		NL "		puts(\"OK\");"
		NL "	return 0;"
		NL "}"
		NL;

	require("cc/cc", logdepth, fatal);

	report("Checking for ftime()... ");
	logprintf(logdepth, "find_time_ftime: trying to find ftime...\n");
	logdepth++;

	if (try(logdepth, test_c, NULL, NULL)) {
		put("libs/time/ftime/presents", strue);
		report("OK\n");
		return 0;
	}
	put("libs/time/ftime/presents", sfalse);
	report("not found.\n");

	return 1;
}

