/*
    libporty - dynamic, splitting fifo
    Copyright (C) 2010..2012  Tibor 'Igor2' Palinkas

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

    Project page: http://repo.hu/projects/libporty
    Author: libporty@igor2.repo.hu
*/

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "buf.h"

#define GROW 256

static char *strclone(const char *str)
{
	int len;
	char *s;

	len = strlen(str);
	s   = malloc(len+1);
	memcpy(s, str, len+1);
	return s;
}

P_buf_t *P_buf_init(const char *sep)
{
	P_buf_t *b;
	b = malloc(sizeof(P_buf_t));
	b->alloced = 0;
	b->used    = 0;
	b->sep     = sep == NULL ? NULL : strclone(sep);
	b->next    = sep == NULL ? NULL : P_buf_init(NULL);
	b->last    = b->next;
	b->prev    = NULL;
	return b;
}

static void append(P_buf_t **b, const char *in, int len)
{
	if ((*b)->used + len + 1 >= (*b)->alloced) {
		(*b)->alloced = (*b)->used + len + GROW;
		(*b) = realloc(*b, (*b)->alloced + sizeof(P_buf_t));
	}
	memcpy((*b)->buf+(*b)->used, in, len);
	(*b)->used += len;
	(*b)->buf[(*b)->used] = '\0';
}

void P_buf_fill(P_buf_t *buff, const char *strin)
{
	const char *end, *start;
	int len;
	P_buf_t *last, *nw, *prev;

	for(end = start = strin; end != NULL; ) {
		end = strpbrk(start, buff->sep);
		if (end != NULL)
			len = end - start;
		else
			len = strlen(start);

		prev = buff->last->prev;
		append(&(buff->last), start, len);
		if (prev != NULL)
			prev->next = buff->last;
		else
			buff->next = buff->last;

		if (end != NULL) {
			last = buff->last;
			nw = P_buf_init(NULL);

			last->next = nw;
			nw->prev = last;
			buff->last = nw;
		}
		start = end+1;
	}
}

char *P_buf_get(P_buf_t *buff)
{
	P_buf_t *b;
	b = buff->next;
	if (b->next == NULL)
		return NULL;
	buff->next = b->next;
	b->next->prev = NULL;
	b->next = NULL;
	b->prev = NULL;

	return b->buf;
}

void P_buf_free(char *str)
{
	P_buf_t *b, f;
	int offs;

	offs = (char *)&(f.buf) - (char *)&f;
	b = (P_buf_t *)(str - offs);
	assert(b->sep == NULL);
	assert(b->next == NULL);
	assert(b->prev == NULL);

	free(b);
}


void P_buf_uninit(P_buf_t *buff)
{
	char *line;
	while(1) {
		line = P_buf_get(buff);
		if (line == NULL)
			break;
		P_buf_free(line);
	}
	free(buff);
}

